/*
*  Bookmark Duplicate Detector
*
* Author: Stphane BERTIN
* http://bookmarkdd.mozdev.org
*
* Originally created by Paradox v0.0.1
* Ported to Firefox 1.5 by Stefano Caselli v0.0.5 and v0.0.7
* Since v0.0.7 Stphane BERTIN is the developer
*
* Principal changes:
* 0.2.2: Support for firefox 1.0 added
* 0.3: Add "Search Duplicate Bookmark" function
* 0.4: Rewrite to avoid conflict with others extensions, searchBar feature improved
* 0.5: add autoDelete duplicates, improve customization of menus, add Move button
* 0.6: add support for addBookmarkHere extension, change bookmarks folder, export bookmarks
*
* TODO: 
*/
//----------------------------------------------------------------------------

//Used to include only one time bookmark.js and avoid error message about already specified constant
try{
    if(ADD_BM_DIALOG_FEATURES) {}
} catch(e){
    var loader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
		    .getService(Components.interfaces.mozIJSSubScriptLoader);
    loader.loadSubScript("chrome://browser/content/bookmarks/bookmarks.js");
}

//Used to include only one time UserPrefs.js and avoid error message about already specified constant
try{
    if(UserPrefs) {}
} catch(e){
    var loader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
		    .getService(Components.interfaces.mozIJSSubScriptLoader);
    loader.loadSubScript("chrome://duplicates/content/UserPrefs.js");
}

//Init Bookmarks.js Service
initServices();
initBMService();

//gNC_NS is defined since Firefox 1.5 --> NC_NS === undefined
//NC_NS is defined for Firefox 1.0 --> gNC_NS === undefined

const gNCNS = "http://home.netscape.com/NC-rdf#";


var addBookmarkHere = UserPrefs.getPref("bool","onLoad.addBookmarkHere");


var Bookmarkdd = {

    //Default version is Firefox 1.5 :-)
    version: "1.5",
    // Used to DEBUG
    DEBUG: false,
    userResponse: false,

    // Used by drag&drop
    insertAndCheckURL: function (aAction, aSelection, aTarget, aTargetIndex)
    {
	var DuplicatesBundle = document.getElementById('duplicates-strings');
	if (!DuplicatesBundle) {
	    // Used by BookmarksPanelOverlay
	    DuplicatesBundle = top.document.getElementById('duplicates-strings');
	}//Bookmarkdd.searchStringLocalisation("SearchDup-none")
	var isValid = BookmarksUtils.isSelectionValidForInsertion(aSelection, aTarget);
	if (!isValid) {
	    SOUND.beep();
	    return false;
	}
	if (aAction == "drag" || aAction == "paste" || (aAction == "newbookmark" && addBookmarkHere)){
	    var itemValue = "";
	    var type = aSelection.type[0];
	    //var isContainer = aSelection.isContainer[0];
	    //alert("type "+type+" isContainer "+isContainer);
	    if (aSelection.prop && type !="Livemark"){ //TODO: not support Folder ! && !isContainer
		var removedProp = aSelection.prop[0];
		itemValue = removedProp[0][2];
		//var urlFeed= BookmarksUtils.getProperty(aSelection.item[0], gNC_NS+"FeedURL" );
		//alert("type="+type+" UrlFeed: "+urlFeed);
	    } else {
		itemValue = BMDS.GetTarget(aSelection.item[0], RDF.GetResource(gNCNS+"FeedURL"), true);
		if (!itemValue) itemValue = BMDS.GetTarget(aSelection.item[0], RDF.GetResource(gNCNS+"URL"), true);
		//itemValue = BMDS.GetTarget(aSelection.item[0], RDF.GetResource(gNCNS+"URL"), true);
	    }
	    try { //Folder hasn't URL so it throw an error and continue after the catch
		var url = itemValue.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
		if (Bookmarkdd.DEBUG) Bookmarkdd.myDump("insertAndCheckURL + aAction: "+aAction+"\nurl = "+url);

		var isDuplicate = false;
		var rssFeed = /(.rss)$/i;	//Verify if url is a FeedURL
		var urlFeed = rssFeed.exec(url);
		if (!urlFeed) {
		    isDuplicate = Bookmarkdd.isBookmarked(url);
		} else {
		    isDuplicate = Bookmarkdd.isBookmarked(url,url);
		}

		if(isDuplicate) {
		    if (DuplicatesBundle){
			var message = DuplicatesBundle.getString("alreadyIn-Folder")+"\n";
			message += isDuplicate.parents.join("\n")+"\n";
			message += DuplicatesBundle.getString("alreadyIn-Confirm");
		    } else {
			var message = Bookmarkdd.searchStringLocalisation("alreadyIn-Folder")+"\n";
			message += isDuplicate.parents.join("\n")+"\n";
			message += Bookmarkdd.searchStringLocalisation("alreadyIn-Confirm");
		    }
		    if(! BddConfirm(message, isDuplicate.RDFnodes)) return false;
		}
	    } catch(e){
		// if action== paste and isContainer and !isLivemark --> folder -->error
		if (Bookmarkdd.DEBUG) Bookmarkdd.myDump("insertAndCheckURL + error with action: "+aAction);
	    }
	} else if (Bookmarkdd.DEBUG) {
	    Bookmarkdd.myDump("DEBUG: Bookmarkdd.insertAndCheckURL + action not supported: "+aAction);
	    Bookmarkdd.myDump("AddBookmarkHere="+addBookmarkHere);
	}

	BookmarksUtils.insertSelection(aAction, aSelection, aTarget, aTargetIndex);
	BookmarksUtils.flushDataSource();
	if (Bookmarkdd.version == "1.5") BookmarksUtils.refreshSearch();

	return true;
    },


    // Verify if the specified URL is already in Bookmark
    // If URL already in Bookmark function return an Array of Parent folder
    isBookmarked: function (aUrl,aUrlFeed)
    {
	var DuplicatesBundle = document.getElementById('duplicates-strings');

	if ( ! BMSVC || ! BMDS || ! RDF || ! gNCNS ) {
	    alert(DuplicatesBundle.getString("flagrantError-Alert"));
	    return false; 
	}

	var urlArcID = RDF.GetResource(gNCNS+"ID");
	var FeedArc = RDF.GetResource(gNCNS+"FeedURL");
	if (!aUrlFeed){
	    var urlLiteral = RDF.GetLiteral(aUrl);
	    var bmResource = BMSVC.GetSources(RDF.GetResource(gNCNS+"URL"), urlLiteral, true);
	} else {
	    var urlLiteral = RDF.GetLiteral(aUrlFeed);
	    var bmResource = BMSVC.GetSources(RDF.GetResource(gNCNS+"FeedURL"), urlLiteral, true);
	}

	var count = 0;
	var URLFound = new URLConstructor();

	while (bmResource.hasMoreElements()) {
	    var DuplicateURLresource=bmResource.getNext();
	    var URLParent = BMSVC.getParent(DuplicateURLresource);
	    if(URLParent) {
		//A RssFeed can't contain another RssFeed
		//URLParent can't have a FeedURL if the child has a FeedURL (aUrlFeed is defined)
		if (!aUrlFeed){
		    //If parent is a RssFeed (targetFeed!=null) --> the new bookmark is not a duplicate.
		    var targetFeed = BMDS.GetTarget(URLParent, FeedArc, true);
		    if (targetFeed) continue;
		}

		URLFound.parents[count] = this.findPath(DuplicateURLresource);
		//Store RDF node ID for Bookmark manipulation: URLFound.RDFnodes[count].Value =  rdf:#$...
		URLFound.RDFnodes[count] = BMDS.GetTarget(DuplicateURLresource.QueryInterface(kRDFRSCIID), urlArcID, true);
		if(!URLFound.RDFnodes[count]) {
		    // If null it means that the Resource is not already include within RDF document
		    // and DuplicateURLresource is a RDF Node resource.
		    URLFound.RDFnodes[count] = DuplicateURLresource; //DuplicateURLresource.Value = rdf:#$...
		}
		count++;
	    }
	}

	if (count>0) return URLFound;
	return false;
    },

    // Replace addBookmark to verify if URL is duplicated
    addBookmarkURL: function (aURL, aTitle, aCharset, aIsWebPanel, aDescription)
    {
	var DuplicatesBundle = document.getElementById('duplicates-strings');
	var dArgs = {
		name: aTitle,
		url: aURL,
		charset: aCharset,
		bWebPanel: aIsWebPanel,
		description: aDescription
	}

	var isDuplicate = Bookmarkdd.isBookmarked(aURL);
	if(isDuplicate) {
	    var message = DuplicatesBundle.getString("alreadyIn-Folder")+"\n";
	    message += isDuplicate.parents.join("\n")+"\n";
	    message += DuplicatesBundle.getString("alreadyIn-Confirm");
	    if(! BddConfirm(message, isDuplicate.RDFnodes)) return false;
	}

	if(Bookmarkdd.version == "1.5") {
	    return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dArgs);
	} else if (Bookmarkdd.version == "1.0") {
	    return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", "centerscreen,chrome,dialog,resizable,dependent", aTitle,aURL,null,aCharset,null,null,aIsWebPanel);
	}

	return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dArgs);
    },

    addLivemark: function (aURL, aFeedURL, aTitle, aDescription)
    {
	var DuplicatesBundle = document.getElementById('duplicates-strings');
	var dArgs = {
	    name: aTitle,
	    url: aURL,
	    bWebPanel: false,
	    feedURL: aFeedURL,
	    description: aDescription
	};
	try {
	    var toolbarFolderURI = BMDS.getBookmarksToolbarFolder().Value;
	    dArgs.folderURI = toolbarFolderURI;
	} catch (e) { 
	}
	//window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dArgs);

	var isDuplicate = Bookmarkdd.isBookmarked(aURL, aFeedURL);
	if(isDuplicate) {
	    var message = DuplicatesBundle.getString("alreadyIn-Folder")+"\n";
	    message += isDuplicate.parents.join("\n")+"\n";
	    message += DuplicatesBundle.getString("alreadyIn-Confirm");
	    if(! BddConfirm(message, isDuplicate.RDFnodes)) return false;
	}

	if(Bookmarkdd.version == "1.5") {
	    return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dArgs);
	} else if (Bookmarkdd.version == "1.0") {
	    return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", "centerscreen,chrome,dialog,resizable,dependent", aTitle,aURL,null,null,null,null,false,null,null,null,aFeedURL);
	}

	return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dArgs);
    },

    // Function used to search in ALL Bookmarks which URL are duplicated
    searchDuplicateURL: function ()
    {
	var DuplicatesBundle = document.getElementById('duplicates-strings');
	var urlArc = RDF.GetResource(gNCNS+"URL");
	var FeedArc = RDF.GetResource(gNCNS+"FeedURL");

	var AllBookmarksResources = BMDS.GetAllResources();

	var AllURLs = new Array(), URLDuplicates = new Array();
	var AllURLsTotal = 0, URLdupNum = 0, AllUniqueURLsNumber=0;

	//Bookmark{URL}
	//Livemark{URL, FeedURL} So livemark is also detected by Bookmark target.
	while (AllBookmarksResources.hasMoreElements()) {
	    var urlRes = AllBookmarksResources.getNext().QueryInterface(kRDFRSCIID);

	    //if Parent's Folder is a Livemark don't verify URL go to next Resource
	    var aParent = BMSVC.getParent(urlRes);
	    if (aParent) var ptype = BookmarksUtils.resolveType(aParent);
	    if (ptype == "Livemark") continue;

	    var target = BMDS.GetTarget(urlRes, urlArc, true);
	    //Is urlRes a RSS folder ?
	    var targetFeed = BMDS.GetTarget(urlRes, FeedArc, true);
	    if (targetFeed) {
//var targetURL = target.QueryInterface(kRDFLITIID).Value;
	    	var targetFeedURL = targetFeed.QueryInterface(kRDFLITIID).Value;
		//URLFeed still detected even if it has been removed during same Firefox session
		// so we need to verify if it is really a duplicate Resource
//var FeedDuplicate = Bookmarkdd.isBookmarked(targetURL,targetFeedURL);
		var FeedDuplicate = Bookmarkdd.isBookmarked("",targetFeedURL);
		if (FeedDuplicate && FeedDuplicate.parents.length < 2)  continue;
		target = targetFeed;
	    }

	    //var urlArcID   = RDF.GetResource(gNCNS+"ID");
	    //var Node = BMDS.GetTarget(urlRes, urlArcID, true);
	    //if (target && ptype != "Livemark"){
	    if (target){
		var url = target.QueryInterface(kRDFLITIID).Value;
		if(Bookmarkdd.version == "1.5"){
		    if (AllURLs.indexOf(url) == -1){
			    // 1.5 and 1.0 use different variables without interaction
			    AllURLs[AllUniqueURLsNumber] = url;
			    AllUniqueURLsNumber++;
		    } else {
			    if (URLDuplicates.indexOf(url) == -1) {
				    URLDuplicates[URLdupNum] = url;
				    URLdupNum++;
			    }
		    }
		} else { //Bookmarkdd.version == "1.0"
		    // 1.5 and 1.0 use different variables without interaction
		    AllURLs[AllURLsTotal] = url;
		}
		AllURLsTotal++;
	    }

	}
	if  (Bookmarkdd.version == "1.0"){
	    var AllURLnum = AllURLs.length, isDuplicate = false;
	    var rssFeed = /(.rss)$/i;	//Verify if url is a FeedURL
	    for (var countB = 0; countB < AllURLnum;countB++){
		if (!AllURLs[countB]) continue;

		url = AllURLs[countB];
		var urlFeed = rssFeed.exec(url);

		if (!urlFeed) {
		    isDuplicate = Bookmarkdd.isBookmarked(url);
		} else {
		    isDuplicate = Bookmarkdd.isBookmarked(url,url);
		}
		if (isDuplicate && isDuplicate.parents.length > 1){
		    var alreadyIN = false;
		    for (var count=0, URLnum=URLDuplicates.length; count<URLnum; count++){
			if (URLDuplicates[count] == url) alreadyIN = true;
		    }
		    if (!alreadyIN){
			URLDuplicates[URLdupNum] = url;
			URLdupNum++;
		    } else { var removed = AllURLs.splice(countB, 1); } //Remove each duplicate to decrease loop round
		}
	    }
	}

	if (URLdupNum >0 ){
	    URLDuplicates.sort();
	    window.openDialog("chrome://duplicates/content/ManageDuplicates.xul", "bookmarkDD-ManageDuplicates", "chrome,alwaysRaised,resizable,centerscreen,dialog=no", URLDuplicates,URLdupNum,AllURLsTotal);
	} else {
		try {
		    alert(DuplicatesBundle.getString("SearchDup-none"));
		} catch(e) {
		    alert(Bookmarkdd.searchStringLocalisation("SearchDup-none"));
		}
	}
    },


    // Return a chain of URL Parent Folder Name
    findPath: function (URLresource)
    {
	var ParentResource = URLresource;
	var PathArray = new Array();
	var boucle = true, ParentName;
/*XXX Maybe the following is faster
	//Extract from bookmarksTree.xml
	var chain = BMSVC.getParentChain(aResource);
	Bookmarkdd.myDump("Chain length= "+chain.length+"\n");
	for (var i=0; i<chain.length; i++) {
	    var rParent = chain.queryElementAt(i, kRDFRSCIID);
	    Bookmarkdd.myDump(i+":"+BookmarksUtils.getProperty(rParent, gNC_NS+"Name"));
	}
*/
	while (boucle){
	    ParentResource = BMSVC.getParent(ParentResource);
	    if (ParentResource) {
		ParentName = BookmarksUtils.getProperty(ParentResource.Value,gNCNS+"Name");
		if (ParentName) PathArray.push(ParentName);
	    } else {
		boucle = false;
	    }
	}
	PathArray.reverse();

	return PathArray.join(" > ");
    },


    //This function is executed when Option panel unload
    VerifyMenuItemStatus: function ()
    {
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
		    .getService(Components.interfaces.nsIWindowMediator);
	var enumerator = wm.getEnumerator("");
	var enumBreak = 0;

	while(enumerator.hasMoreElements()) {
	    var win = enumerator.getNext();
	    // |win| is [Object ChromeWindow] (just like |window|), do something with it
	    // Verify if BookmarksMenu's hidden property is synchronised with UserPrefs
	    var BookmarksMenu=win.document.getElementById("BookmarkDD-context-menu");
	    if(BookmarksMenu) {
		if(!UserPrefs.getPref("bool","BookmarksMenu.addLink")) {
		    BookmarksMenu.setAttribute('hidden',true);
		} else {
		    BookmarksMenu.setAttribute('hidden',false);
		}
		enumBreak++;
	    }
	    // Verify if ToolsMenu's hidden property is synchronised with UserPrefs
	    var ToolsMenu=win.document.getElementById("BookmarkDD-PanelLink");
	    if(ToolsMenu) {
		if(!UserPrefs.getPref("bool","ToolsMenu.addLink")) {
		    ToolsMenu.setAttribute('hidden',true);
		} else {
		    ToolsMenu.setAttribute('hidden',false);
		}
		enumBreak++;
	    }
	    if(enumBreak>0) break;
	}

    },

    //Load preferences that need to be initialized only one times
    loadPreferences: function ()
    {
	try{
	    var menu=document.getElementById("bookmarks-menu");
	    if(!menu) return; //We are not in windowsID="main-window"

	    // Show/Hide Bookmarks menu and Tools menu BookmarkDD's items
	    if(!UserPrefs.getPref("bool","BookmarksMenu.addLink")) 
	    			document.getElementById("BookmarkDD-context-menu").setAttribute('hidden',true);
	    if(!UserPrefs.getPref("bool","ToolsMenu.addLink")) 
	    			document.getElementById("BookmarkDD-PanelLink").setAttribute('hidden',true);


	    //Use of intermediate preference variable to manage changement of AddBookmarkHere option and
	    //avoid problem with Bookmarkdd.addBookmarkURL; Bookmarkdd.addLivemark; which are loaded only by firefox start.
	    addBookmarkHere = UserPrefs.getPref("bool","Miscellaneous.addBookmarkHere");
	    UserPrefs.setPref("bool","onLoad.addBookmarkHere",addBookmarkHere);
	    if(!addBookmarkHere){
		BookmarksUtils.addBookmark = Bookmarkdd.addBookmarkURL;
		BookmarksUtils.addLivemark = Bookmarkdd.addLivemark;
	    }

	} catch(e) {
	    alert(e);
	    return;
	}
	if (Bookmarkdd.DEBUG) Bookmarkdd.myDump("AddBookmarkHere="+addBookmarkHere);

    },


    // With a RDF node generate a Selection used by BookmarksCommand
    createSelection: function (aNode)
    {
	var items = new Array();
	items[0] = aNode;//URLrdfNodes[aItem.value];
	var selection = {item: items, parent:Array(1), length: 1, prop:Array(0)};
	selection.parent[0] = BMDS.getParent(selection.item[0]);
	selection.isContainer = new Array(selection.length); //It is not a container !!

	return selection;
    },

    //Function used to log message within JS Console
    myDump: function (aMessage) 
    {
	var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
				 .getService(Components.interfaces.nsIConsoleService);
	consoleService.logStringMessage("BookmarkDD's message: " + aMessage);
    },

    // Function used to find localized Strings due to restriction of overlay: element in overlay have no ID !!
    // Insertion of <stringbundle id="duplicates-strings"/> can't be called !!!!!
    searchStringLocalisation: function (aLabel){
	var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
		    .getService(Components.interfaces.nsIWindowMediator);
	var enumerator = wm.getEnumerator("");

	while(enumerator.hasMoreElements()) {
	    var win = enumerator.getNext();
	    // |win| is [Object ChromeWindow] (just like |window|), do something with it
	    var duplicatesStrings=win.document.getElementById("duplicates-strings");
	    if(duplicatesStrings) {
		return duplicatesStrings.getString(aLabel);
	    }
	}
	return false;
    }

}//End of Bookmarkdd variable



// Replace function addBookmarkForTabBrowser in browser.js
// Add support for isBookmarked()
function addBookmarkForTabBrowser(aTabBrowser, aBookmarkAllTabs, aSelect)
{
    var DuplicatesBundle = document.getElementById('duplicates-strings');
    var tabsInfo = [];
    var currentTabInfo = { name: "", url: "", charset: null };
    if (Bookmarkdd.version == "1.0" && aBookmarkAllTabs) {
	aSelect = aBookmarkAllTabs;
	if (aSelect) aBookmarkAllTabs = true;
    }

    const activeBrowser = aTabBrowser.selectedBrowser;
    const browsers = aTabBrowser.browsers;
    for (var i = 0, j = 0; i < browsers.length; ++i) {
	var webNav = browsers[i].webNavigation;
	var url = webNav.currentURI.spec;
	var name = "";
	var charSet, description;
	try {
	    var doc = webNav.document;
	    name = doc.title || url;
	    charSet = doc.characterSet;
	    if (Bookmarkdd.version == "1.5") description = BookmarksUtils.getDescriptionFromDocument(doc);
	} catch (e) {
	    name = url;
	}

	var isDuplicate = Bookmarkdd.isBookmarked(url);
	if(isDuplicate) {
	    var messageURL = DuplicatesBundle.getString("followingURL")+"\n";
	    messageURL += url+"\n";
	    messageURL += DuplicatesBundle.getString("inFolder")+"\n";
	    messageURL += isDuplicate.parents.join("\n")+"\n";
	    messageURL += DuplicatesBundle.getString("alreadyIn-Confirm");

	    var message = DuplicatesBundle.getString("alreadyIn-Folder")+"\n";
	    message += isDuplicate.parents.join("\n")+"\n";
	    message += DuplicatesBundle.getString("alreadyIn-Confirm");

	    if(aBookmarkAllTabs) {
		if (! BddConfirm(messageURL, isDuplicate.RDFnodes)) continue;
	    } else if (browsers[i] == activeBrowser) {
		if (! BddConfirm(message, isDuplicate.RDFnodes)) return false;
	    }
	}

	tabsInfo[j] = { name: name, url: url, charset: charSet, description: description };
	if (browsers[i] == activeBrowser) currentTabInfo = tabsInfo[j];
	j++;
    }

    if (tabsInfo.length == 0) {
	alert (DuplicatesBundle.getString("NoNewURL"));
	return false;
    }

    var dialogArgs = currentTabInfo;
    if (Bookmarkdd.version == "1.5" && aBookmarkAllTabs) {
	dialogArgs = { name: gNavigatorBundle.getString("bookmarkAllTabsDefault") };
	dialogArgs.bBookmarkAllTabs = true;
    }


    dialogArgs.objGroup = tabsInfo;
    if(Bookmarkdd.version == "1.5") {
	return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dialogArgs);
    } else if (Bookmarkdd.version == "1.0") {
	return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", "centerscreen,chrome,dialog,resizable,dependent", currentTabInfo.name, currentTabInfo.url, null, currentTabInfo.charset, "addGroup" + (aSelect ? ",group" : ""), tabsInfo);
    }

    return window.openDialog("chrome://browser/content/bookmarks/addBookmark2.xul", "", ADD_BM_DIALOG_FEATURES, dialogArgs);
}

//BookmarkDD uses its own confirm window to replace default icon without overlay icon for all Browser's confirm
function BddConfirm(aText, aRDFnodes){

    //alert("aText "+aText);
    window.openDialog("chrome://duplicates/content/ConfirmOverlay.xul", "BookmarkDD-Confirm", "chrome,modal,alwaysRaised,resizable,centerscreen", aText, aRDFnodes);

    return Bookmarkdd.userResponse;
}

//Verify which Firefox version is used
try {
    if (gNC_NS) Bookmarkdd.version="1.5";
}
catch(e){
    Bookmarkdd.version = "1.0";
}


//Add properties to URLFound Array
function URLConstructor(){
    this.parents = new Array();
    this.RDFnodes = new Array();
}


// Replace BookmarksUtils method by our functions to detect duplicate URL
BookmarksUtils.insertAndCheckSelection = Bookmarkdd.insertAndCheckURL; 
/*if(!addBookmarkHere){
    BookmarksUtils.addBookmark = Bookmarkdd.addBookmarkURL;
    BookmarksUtils.addLivemark = Bookmarkdd.addLivemark;
}*/


// Export bookmarks.html file when configuration path has been changed
function autoExportBookmark(){
    //Do we need to export bookmark file ?
    if ( ! UserPrefs.getPref("bool","BookmarksFile.export") ) return;

    var prefService = Components.classes["@mozilla.org/preferences-service;1"].
    			getService(Components.interfaces.nsIPrefService);
    prefService = prefService.getBranch("browser.");
    var fileName = "";
    try{
	fileName = prefService.getCharPref("bookmarks.file");
    } catch(e) {
	//bookmarks.file doesn't exist when bookmarks.html is in default location
	fileName = Components.classes["@mozilla.org/file/directory_service;1"].
			getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
	fileName.append("bookmarks.html");
	fileName = fileName.path;
	//alert("Bookmarks file will be:\n"+fileName);
    }

    var selection = RDF.GetResource("NC:BookmarksRoot");
    var args = [{ property: gNC_NS+"URL", literal: fileName}];
    BookmarksCommand.doBookmarksCommand(selection, gNC_NS_CMD+"export", args);

    // We only need to export bookmarks when configuration's options changed
    // So next start we don't need to export except configuration is modified
    UserPrefs.setPref("bool","BookmarksFile.export", false);
}
